import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking
import kotlinx.coroutines.supervisorScope
import ru.landgrafhomyak.bgtu.networks0.icmp.pinger_IPv4
import ru.landgrafhomyak.bgtu.networks0.low_level.multithreading.Thread
import ru.landgrafhomyak.bgtu.networks0.low_level.sockets.EpollSocketEventLoop

class EventLoopRoutine(private val loop: EpollSocketEventLoop) : Thread.Routine {
	override fun run() {
		while (true) {
			try {
				this.loop.runUntilError()
			} catch (e: Throwable) {
				e.printStackTrace()
				break // todo remove
			}
		}
	}
}

fun main(argv: Array<String>) {
	try {
		EpollSocketEventLoop().use { loop ->
			Thread(EventLoopRoutine(loop)).use { loopThread ->
				loopThread.start()
				val values = arrayOfNulls<UInt>(argv.size)
				runBlocking {
					launch {
						while (true) {
							for ((addr, time) in (argv zip values)) {
								println(time.toString().padStart(5, ' ') + " ${addr}")
							}
							print("\u001B[${argv.size}A\r")
							delay(1000)
						}
					}

					supervisorScope {
						argv.forEachIndexed { i, addr ->
							launch {
								loop.pinger_IPv4(addr).use { pinger ->
									while (true) {
										values[i] = pinger.ping(1000u)
										delay(1000)
									}
								}
							}
						}
					}
				}
			}
		}
	} catch (e: Throwable) {
		e.printStackTrace()
	}
}